@@ -81,6 +81,9 @@ hostname_is_valid (const gchar *name)
                                  name, G_REGEX_MULTILINE, 0);
 }
 
+/*
+   Taken with a few minor changes from systemd's hostnamed.c,
+*/
 static gchar *
 guess_chassis ()
 {
@@ -97,23 +100,20 @@ guess_chassis ()
 
 #if defined(__i386__) || defined(__x86_64__)
     /* 
-       Taken with a few minor changes from systemd's hostnamed.c,
-       copyright 2011 Lennart Poettering.
+       See the SMBIOS Specification 3.5.0 section 7.4.1 for details about the values listed here:
 
-       See the SMBIOS Specification 3.4.0 section 7.4.1 for
-       details about the values listed here:
-
-       https://www.dmtf.org/sites/default/files/standards/documents/DSP0134_3.4.0.pdf
+       https://www.dmtf.org/sites/default/files/standards/documents/DSP0134_3.5.0.pdf
     */
 
     if (g_file_get_contents ("/sys/class/dmi/id/chassis_type", &filebuf, NULL, NULL)) {
         switch (g_ascii_strtoull (filebuf, NULL, 10)) {
         case 0x3: /* Desktop */
         case 0x4: /* Low Profile Desktop */
-        case 0x5: /* Pizza Box */
         case 0x6: /* Mini Tower */
         case 0x7: /* Tower */
-        case 0xD: /* All in One */
+        case 0xD: /* All in one (i.e. PC built into monitor) */
+        case 0x23: /* Mini PC */
+        case 0x24: /* Stick PC */
             ret = g_strdup ("desktop");
             goto out;
         case 0x8: /* Portable */
@@ -126,7 +126,6 @@ guess_chassis ()
             ret = g_strdup ("handset");
             goto out;
         case 0x11: /* Main Server Chassis */
-        case 0x17: /* Rack Mount Chassis */
         case 0x1C: /* Blade */
         case 0x1D: /* Blade Enclosure */
             ret = g_strdup ("server");
@@ -138,9 +137,55 @@ guess_chassis ()
         case 0x20: /* Detachable */
             ret = g_strdup ("convertible");
             goto out;
+        case 0x21: /* IoT Gateway */
+        case 0x22: /* Embedded PC */
+            ret = g_strdup ("embedded");
+            goto out;
         }
     }
 #endif
+    /* We only list the really obvious cases here as the ACPI data is not really super reliable.
+     *
+     * See the ACPI 5.0 Spec Section 5.2.9.1 for details:
+     *
+     * http://www.acpi.info/DOWNLOADS/ACPIspec50.pdf
+     */
+    if (g_file_get_contents ("/sys/firmware/acpi/pm_profile", &filebuf, NULL, NULL)) {
+        switch (g_ascii_strtoull (filebuf, NULL, 10)) {
+
+        case 1: /* Desktop */
+        case 3: /* Workstation */
+        case 6: /* Appliance PC */
+            ret = g_strdup ("desktop");
+            goto out;
+
+        case 2: /* Mobile */
+            ret = g_strdup ("laptop");
+            goto out;
+
+        case 4: /* Enterprise Server */
+        case 5: /* SOHO Server */
+        case 7: /* Performance Server */
+            ret = g_strdup ("server");
+            goto out;
+
+        case 8: /* Tablet */
+            ret = g_strdup ("tablet");
+            goto out;
+        }
+    }
+
+    /* Query Device Tree for chassis type
+     *
+     * Note that the Devicetree specification uses the very same vocabulary
+     * of chassis types as we do, hence we do not need to translate these types:
+     *
+     * https://github.com/devicetree-org/devicetree-specification/blob/master/source/chapter3-devicenodes.rst */
+    if (g_file_get_contents ("/proc/device-tree/chassis-type", &filebuf, NULL, NULL)) {
+        ret = g_strdup (filebuf);
+        goto out;
+    }
+
   out:
     g_free (filebuf);
     return ret;
